/* Floy.cc
**
** DCFloys
** (c)2001 Paul Boese a.k.a. Axlen
** Based on Floys Java Applet written by Ariel Dolan http:://www.aridolan.com/
**
*/

#include "headers.h"
#include <math.h>
#include "Floy.h"

Floy::Floy(int numf, int maxn, int id, float wid, float hgt, float dep) {
		
	fid = id;
	width = wid;
	height = hgt;
	depth = dep;
	maxnb = maxn;
	margin = 30.0f;    // bounding margin
	v0 = 30.0f;        // initial velocity
/*		
	numnb = 1;         // number of neighbors
	revdist = 80.0f;   // reverse distance
	acc = 0.05f;       // acceleration
	attcen = 0.01f;    // attraction to center
	maxv = 3.0f;       // maximum velocity
	bouncev = 0.4f;    // bounce velocity
*/
	type = 0;          // normal floy=0, stranger=1
	energy = 20;       // energy (stranger)

	xtail = ytail = ztail = 0;
	
	x = (float)(width/2 - randnum((int)width));
	y = (float)(height/2 - randnum((int)height));
	z = (float)(depth/2 - randnum((int)depth));
	vx = (float)(v0/2 - randnum((int)v0))/10.0f;
	vy = (float)(v0/2 - randnum((int)v0))/10.0f;
	vz = (float)(v0/2 - randnum((int)v0))/10.0f;

	neighbors = (Floy**) malloc(maxnb * sizeof(Floy*));

}

void Floy::params(int num_nb, float rev_dist, 
					float accel, float att_cen,
					float max_v, float bounce_v) {

	numnb = num_nb;
	revdist = rev_dist;
	acc = accel;
	attcen = att_cen;
	maxv = max_v;
	bouncev = bounce_v;

}

float Floy::dist(Floy* pnb) {
	float d, d1, d2, d3;
	d1 = x - pnb->x;
	d2 = y - pnb->y;
	d3 = z - pnb->z;
	d = fabs(d1*d1+d2*d2+d3*d3);
	return d;
}

void Floy::GetNeighbors(){
	int i, j, k;
	Floy* pnb;

	for (i=0; i<numnb; i++){
		pnb = neighbors[i];
		for (j=0; j<numnb; j++){
			if (dist(pnb->neighbors[j]) < dist(pnb))
				neighbors[i] = pnb->neighbors[j];
			if (pnb->neighbors[j]->type == 1)
				neighbors[i] = pnb->neighbors[j];
		}
	}

	for (k=0; k<numnb;k++){
		if (k == fid) neighbors[k] = neighbors[k]->neighbors[0];
	}
}

void Floy::Process(){
	float rev, d;
	int i;

	xtail = x;
	ytail = y;
	ztail = z;
	rev = -1.0f;

	for (i=0; i<numnb; i++){

		d = dist(neighbors[i]);
		if (d == 0.0f){
			rev = 0.0f;
		} else if (d < revdist){
			if (neighbors[i]->type != type){
				if (type == 1){
					energy--;
					rev = -20.0f;
				} else {
					rev = 20.0f;
				}
			} else {
				rev = -1.0f;
			}
		} else {
			if (type == 1){
				rev = 0.0f;
			} else if (neighbors[i]->type == 1){
				rev = 20.0f;
			} else {
				rev = 1.0f;
			}
		}

		if (x < neighbors[i]->x){
			vx += acc*rev;
		} else {
			vx -= acc*rev;
		}

		if (y < neighbors[i]->y){
			vy += acc*rev;
		} else {
			vy -= acc*rev;
		}

		if (z < neighbors[i]->z){
			vz += acc*rev;
		} else {
			vz -= acc*rev;
		}
	}
	
	if (vx > maxv) vx = maxv;
	if (vx < -maxv) vx = -maxv;
	if (vy > maxv) vy = maxv;
	if (vy < -maxv) vy = -maxv;
	if (vz > maxv) vz = maxv;
	if (vz < -maxv) vz = -maxv;

	if (x < -(width/2)) vx = bouncev;
	if (x > (width/2)) vx = -bouncev;
	if (y < -(height/2)) vy = bouncev;
	if (y > (height/2)) vy = -bouncev;
	if (z < -(depth/2)) vz = bouncev;
	if (z > (depth/2)) vz = -bouncev;

	if (type == 0){
		if (x < 0.0f) vx += attcen;
		if (x > 0.0f) vx -= attcen;
		if (y < 0.0f) vy += attcen;
		if (y > 0.0f) vy -= attcen;
		if (z < 0.0f) vz += attcen;
		if (z > 0.0f) vz -= attcen;
	}

	x += vx;
	y += vy;
	z += vz;
	if (energy < 1){
		type = 0;
		energy = 20;
	}

	// This little bit of magic lets us find rotations that
	// we need to apply to the object so that it points in
	// the direction it is moving.
	//
	// Unfortunately this method suffers from gimbel lock,
	// or I just simply don't understand what's happening
	// very well.  More likely the later.
	// 
	// get vector
	float srv;
	a = x - xtail;
	b = y - ytail;
	c = z - ztail;
	srv = sqrtf(a*a + b*b + c*c);
	// now the direction cosines
	rx = (a / srv);
	ry = (b / srv);
	rz = (c / srv);
	
}

void Floy::Draw(){
	;
}

void Floy::Carp(){

	printf("[%02d][%+04d %+04d %+04d][%+02d %+02d %+02d][%08x][%08x][%08x]\n",
			(int)fid, (int)x, (int)y, (int)z,
			(int)vx, (int)vy, (int)vz,
			(void*)this, (void*)neighbors[0], (void*)neighbors[1]);
}


/* vi:tabstop=4
*/

